@@ -5,6 +5,7 @@ extern "C"{
 void rIW(
         double *nu,      // df
         double *V,       // inverse scale matrix
+		double *CMP,      // conditional submatrix 
         int *dimP,   // dimension
         int *splitP,   // dimension
         int *n,          // number of samples
@@ -15,13 +16,18 @@ void rIW(
 
   int i, j, dim, split, cnt;
   css *GS;
-  cs *G, *Gsamp;
+  cs *G, *Gsamp, *CM;
 
   dim = dimP[0];
   split = splitP[0];
 
   G = cs_spalloc(dim, dim, dim*dim, true, false);
-
+	if(split==-999){
+	  CM = cs_spalloc(1, 1,1, true, false);
+	}else{
+      CM = cs_spalloc(dim-split, dim-split, (dim-split)*(dim-split), true, false);
+	}
+	
   cnt=0;
   for (i = 0 ; i < dim; i++){
     G->p[i] = i*dim;
@@ -46,8 +52,19 @@ void rIW(
       cs_spfree(Gsamp);
     }
   }else{          /* conditional inverse wishart */
+	  cnt=0;
+	  for (i = split ; i < dim; i++){
+		  CM->p[i-split] = (i-split)*(dim-split);
+		  for (j = split ; j < dim; j++){
+			  CM->i[cnt] = j-split;
+			  CM->x[cnt] = CMP[cnt];
+			  cnt++;
+		  }
+	  }
+	  CM->p[(dim-split)] = (dim-split)*(dim-split);
+	  cnt=0;
     for (i = 0 ; i < n[0]; i++){
-      Gsamp =  cs_rCinvwishart(G, nu[0], split);
+      Gsamp =  cs_rCinvwishart(G, nu[0], split, CM);
       for (j = 0 ; j < dim*dim; j++){
         output[cnt] = Gsamp->x[j];
         cnt++;
@@ -60,6 +77,7 @@ void rIW(
         
 
   cs_spfree(G);
+  cs_spfree(CM);
   cs_sfree(GS);
 
 }
